﻿using System;
using System.Collections.Generic;
using System.Text;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Utils;
using System.Drawing;
using nicoAE.Plugin.Controls;
using nicoAE.Plugin.Property;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace SamplePlugin.Effect
{
    /// <summary>
    /// イメージの輝度を変更します。
    /// </summary>
    public class Brightness : IEffect
    {
        #region IEffect メンバ

        /// <summary>
        /// エフェクトの名前を取得します。
        /// </summary>
        public string PluginName
        {
            get
            {
                return "輝度変換";
            }
        }

        /// <summary>
        /// 所属するカテゴリを取得します。
        /// </summary>
        public string Category
        {
            get
            {
                //ここで設定した名前のメニューのサブメニューとして表示されます。
                //無理して元々あるカテゴリにする必要はありませんが、かといって全て個別の
                //カテゴリにすると使用者が混乱するので注意しましょう。
                return "色調変換";
            }
        }

        /// <summary>
        /// プリセットのプロパティが存在するかどうかを取得します。
        /// </summary>
        public bool PresetProperty
        {
            get
            {
                //デフォルトのプロパティ以外にキーフレームを打つ必要がある場合は
                //trueを返して下さい。
                return false;
            }
        }

        /// <summary>
        /// このエフェクトが対応するデータを示すMediaTypeを取得します。
        /// </summary>
        public MediaType CorrespondingType
        {
            get
            {
                //基本的に画像への処理をする場合は MediaType.Image | MediaType.Video、
                //音声への処理をする場合は MediaType.Audioを返します。
                return MediaType.Image | MediaType.Video;
            }
        }

        /// <summary>
        /// ユーザーが設定する必要があるプロパティをいじるためのコントロールを返します。
        /// </summary>
        /// <returns>プロパティを設定するためのコントロール。</returns>
        public IPropertyEditControl[] GetControls()
        {
            //ある程度はnicoAE.Plugin.Controlにあるので、作るのが
            //めんどくさい場合はそちらを使って下さい。
            return new IPropertyEditControl[] { new Int32EditControl("上限"), new Int32EditControl("下限") };
        }

        /// <summary>
        /// プロパティ設定用コントロールにセットするデフォルトのプロパティを返します。
        /// し、同一のものにセットされます。
        /// </summary>
        /// <returns>デフォルトの</returns>
        public IEffectProperty[] GetDefaultProperty()
        {
            //コントロールの順番とプロパティの順番は一致しなくてもかまいません。
            //キーフレームを打つ必要がない場合、出来るだけプリセットは使わずここだけですませましょう。
            //プロパティはIEffectProperty.PropertyNameとIPropertyEditControl.PropertyNameを比較
            return new IEffectProperty[] { new Int32Property("上限", 255, 255, 0), new Int32Property("下限", 0, 255, 0) };
        }

        /// <summary>
        /// プリセットのプロパティがある場合、ここでKeyFrameを返します。
        /// indexには、ユーザーがプロパティをセットしたいフレームの位置が入ります。
        /// </summary>
        /// <param name="index">ユーザーがプロパティをセットしたいフレームの位置。</param>
        /// <param name="propertyName">プロパティの名前。本体からは1つずつ要求され、要求するプロパティの名前がここに入ります。</param>
        /// <returns>プリセットのキーフレーム。</returns>
        public KeyFrame[] GetPresetProperty(int index, string propertyName)
        {
            //PresetPropertyでfalseを返している場合は使用されません。入力されたpropertyNameに対して出力する
            //プリセットのプロパティがない場合はnullでもかまいません。
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// エフェクトの初期化処理をここに書きます。
        /// </summary>
        /// <param name="host">プロジェクトの情報がまとめられたインターフェース。</param>
        /// <param name="info">処理を実行するデータの情報。</param>
        public void Initialize(IHost host, MediaInfo info)
        {
            //特に処理することがない場合は空でかまいません。
        }

        /// <summary>
        /// オーディオにエフェクト処理を掛けます。
        /// </summary>
        /// <param name="audio">処理を掛ける元のデータ。フレーム単位に切り出されたヘッダなしのオーディオデータです。
        /// ここで入力されるオーディオのフォーマットはReferenceData.WaveFormatと同じです。</param>
        /// <param name="property">該当するフレーム位置のプロパティ。</param>
        /// <returns>エフェクトを掛けられたデータ。</returns>
        public byte[] ProcessingAudio(byte[] audio, IEffectProperty[] property)
        {
            //CorrespondingTypeでMediaType.Audioを返してない場合は使用されません。
            //エフェクトを掛ける場合、入力された配列の長さから変更しないで下さい。
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// イメージにエフェクト処理を掛けます。
        /// </summary>
        /// <param name="image">処理を掛ける元のデータ。形式は32bitARGBです。</param>
        /// <param name="property">該当するフレーム位置のプロパティ。</param>
        /// <returns>エフェクトを掛けられたデータ。</returns>
        public Bitmap ProcessingImage(Bitmap image, IEffectProperty[] property)
        {
            //CorrespondingTypeでMediaType.Audioを返した場合は使用されません。
            //また、IViolateImageEffectを実装している場合も使用されません。
            //ここではimageのサイズから変更しないで下さい。また、イメージの形式は32bitARGBで出力して下さい。
            int max = 0;
            int min = 0;
            //ここで与えられるプロパティの並びは不定です。swich文などで的確に取り出して下さい。
            for (int i = 0; i < property.Length; i++)
            {
                switch (property[i].PropertyName)
                {
                    case "上限":
                        {
                            max = ((Int32Property)property[i]).TypeSafeValue;
                            break;
                        }
                    case "下限":
                        {
                            min = ((Int32Property)property[i]).TypeSafeValue;
                            break;
                        }
                }
            }
            //出来る限り高速に処理するために、処理が不要な場合は何もせずにimageをそのまま返して下さい。
            if (max != 255 || min != 0)
            {
                //Bitmap.SetPxel()を使うよりもbyte配列に書き出して処理する方が高速に処理できます。
                BitmapData bd = image.LockBits(new Rectangle(new Point(), image.Size), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
                byte[] data = new byte[image.Width * image.Height * 4];
                Marshal.Copy(bd.Scan0, data, 0, data.Length);
                for (int i = 0; i < data.Length; i += 4)
                {
                    int t = ((max - min) * data[i]) / 255 + min;
                    if (t > 255)
                    {
                        data[i] = 255;
                    }
                    else if (t < 0)
                    {
                        data[i] = 0;
                    }
                    else
                    {
                        data[i] = (byte)t;
                    }
                    t = ((max - min) * data[i + 1]) / 255 + min;
                    if (t > 255)
                    {
                        data[i + 1] = 255;
                    }
                    else if (t < 0)
                    {
                        data[i + 1] = 0;
                    }
                    else
                    {
                        data[i + 1] = (byte)t;
                    }
                    t = ((max - min) * data[i + 2]) / 255 + min;
                    if (t > 255)
                    {
                        data[i + 2] = 255;
                    }
                    else if (t < 0)
                    {
                        data[i + 2] = 0;
                    }
                    else
                    {
                        data[i + 2] = (byte)t;
                    }
                }
                Marshal.Copy(data, 0, bd.Scan0, data.Length);
                image.UnlockBits(bd);
            }
            return image;
        }

        #endregion

        #region IDisposable メンバ

        /// <summary>
        /// オブジェクトを破棄するときに呼ばれます。
        /// </summary>
        public void Dispose()
        {
            //Disposeすべきものを使用した場合、ここで必ずDisposeして下さい。
            //このメソッドはいつ呼ばれるかわからないので、想定外の場合でも例外をとばさず、正常に
            //破棄できるように設計して下さい。
        }

        #endregion
    }
}
