﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using nicoAE.Plugin.Controls;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Utils;

namespace SamplePlugin
{
    /// <summary>
    /// DoublePropertyを編集するコントロールです。
    /// </summary>
    //ここではPropertyEditSkeletonを使用します。
    public partial class DoubleEditControl : PropertyEditSkeleton, IPropertyEditControl, IScalingTimeline, ISynchronizeKeyFrame
    {
        /// <summary>
        /// このクラスのコンストラクタ。
        /// </summary>
        /// <param name="name">プロパティの名前。</param>
        public DoubleEditControl(string name)
        {
            InitializeComponent();
            n = name;
            label1.Text = n;
        }

        string n = "";
        double clickPoint = 0;
        bool move = false;
        bool moved = false;
        DoubleProperty def = null;

        #region IPropertyEditControl メンバ

        /// <summary>
        /// プロパティの名前を取得します。
        /// </summary>
        public string PropertyName
        {
            get
            {
                return n;
            }
        }

        /// <summary>
        /// コントロールのSplitterの位置を取得、または設定します。
        /// </summary>
        public override int SplitterDistance
        {
            //必要がなければoverrideする必要はありません。
            //ここでは、label1とtextBox1の位置やサイズを調整します。
            get
            {
                return base.SplitterDistance;
            }
            set
            {
                textBox1.Left = label2.Left = value - textBox1.Width - 3;
                label1.Width = textBox1.Left - 3;
                base.SplitterDistance = value;
            }
        }

        /// <summary>
        /// フレームカーソルの位置を取得、または設定します。
        /// </summary>
        public override int CursorPoint
        {
            //必要がなければoverrideする必要はありません。
            //ここではフレームの位置に応じてtextBox1の値を変化させます。
            get
            {
                return base.CursorPoint;
            }
            set
            {
                base.CursorPoint = value;
                textBox1.Text = label2.Text = ((DoubleProperty)GetProperty(value)).TypeSafeValue.ToString("f" + def.Digit.ToString());
                label2.Visible = true;
                textBox1.Visible = false;
            }
        }

        /// <summary>
        /// デフォルトのプロパティを設定します。
        /// </summary>
        /// <param name="property">設定するプロパティ。</param>
        public void SetDefaultProperty(IEffectProperty property)
        {
            def = (DoubleProperty)property;
        }

        /// <summary>
        /// 指定された位置のプロパティを取得します。
        /// </summary>
        /// <param name="index">取得する位置。</param>
        /// <returns>取得したプロパティ。</returns>
        public IEffectProperty GetProperty(int index)
        {
            KeyFrame frame = base.GetKeyFrame(index);
            if (frame != null)
            {
                if (frame.Type == KeyFrameType.Changeable && index != frame.Frame)
                {
                    KeyFrame next = base.GetNextKeyFrame(frame.Frame);
                    if (next != null)
                    {
                        return new DoubleProperty(n, Math.Round(PluginUtils.CalcParameter(frame.Frame, ((DoubleProperty)frame.Property).TypeSafeValue, next.Frame, ((DoubleProperty)next.Property).TypeSafeValue, index), def.Digit, MidpointRounding.ToEven), def.Maximum, def.Minimum, def.Digit);
                    }
                    else
                    {
                        return frame.Property;
                    }
                }
                else
                {
                    return frame.Property;
                }
            }
            else
            {
                return def;
            }
        }

        #endregion

        /// <summary>
        /// textBox1でEnterが押された場合、キーフレームを設定、またはプロパティを変更します。
        /// </summary>
        private void textBox1_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                double val;
                if (double.TryParse(textBox1.Text, out val))
                {
                    if (val >= def.Maximum)
                    {
                        val = def.Maximum;
                        textBox1.Text = def.Maximum.ToString();
                    }
                    else if (val <= def.Minimum)
                    {
                        val = def.Minimum;
                        textBox1.Text = def.Minimum.ToString();
                    }
                    base.SetProperty(new DoubleProperty(n, Math.Round(val, def.Digit, MidpointRounding.ToEven), def.Maximum, def.Minimum, def.Digit));
                    label2.Text = textBox1.Text;
                    label2.Visible = true;
                    textBox1.Visible = false;
                }
            }
        }

        /// <summary>
        /// label1がクリックされた場合、OnEnter()を呼んで該当するエフェクトを選択状態にします。
        /// </summary>
        private void label1_Click(object sender, EventArgs e)
        {
            OnEnter(e);
        }

        /// <summary>
        /// label2がクリックされた場合、label2をハイドしてtextBox1を表示します。
        /// </summary>
        private void label2_MouseClick(object sender, MouseEventArgs e)
        {
            textBox1.Visible = true;
            label2.Visible = false;
            textBox1.Focus();
        }

        /// <summary>
        /// マウス移動に対する値変化を有効にし、リアルタイムプレビューを有効にします。
        /// </summary>
        private void label2_MouseDown(object sender, MouseEventArgs e)
        {
            OnEnter(EventArgs.Empty);
            move = true;
            moved = false;
            clickPoint = double.Parse(label2.Text) - e.X;
            base.StartRealtimePreview();
        }

        /// <summary>
        /// moveがtrueで、マウスが動かされた場合、その移動量に応じて0.1単位で変化させます。
        /// </summary>
        private void label2_MouseMove(object sender, MouseEventArgs e)
        {
            if (move)
            {
                double t = clickPoint + e.X * 0.1;
                if (t > def.Maximum)
                {
                    t = def.Maximum;
                }
                else if (t < def.Minimum)
                {
                    t = def.Minimum;
                }
                t = Math.Round(t, def.Digit, MidpointRounding.ToEven);
                label2.Text = t.ToString();
                moved = true;
                base.SetProperty(new DoubleProperty(n, t, def.Maximum, def.Minimum, def.Digit));
            }
        }

        /// <summary>
        /// リアルタイムプレビューを無効にした後、movedがtrueの場合、値を確定させ、falseの場合はlabel2をハイドし、textBox1を表示します。
        /// </summary>
        private void label2_MouseUp(object sender, MouseEventArgs e)
        {
            move = false;
            base.EndRealtimePreview();
            if (moved)
            {
                base.SetProperty(new DoubleProperty(n, double.Parse(label2.Text), def.Maximum, def.Minimum, def.Digit));
                textBox1.Text = label2.Text;
            }
            else
            {
                label2_MouseClick(sender, e);
            }
        }
    }
}
