﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Utils;
using System.Drawing;
using System.IO;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;

namespace SamplePlugin
{
    /// <summary>
    /// 連番イメージを入力します。
    /// </summary>
    public class SequentialImageInput : IMediaInput
    {
        string startFile = "";
        string[] files = null;
        Size baseSize = new Size(1, 1);
        IHost ho = null;

        #region IMediaInput メンバ

        /// <summary>
        /// プラグインの名前を取得します。
        /// </summary>
        public string PluginName
        {
            get
            {
                return "連番イメージ";
            }
        }

        /// <summary>
        /// 対応する拡張子を取得します。
        /// </summary>
        public string CorrespondingType
        {
            get
            {
                //書式は"表示名|拡張子"を一セットとし、拡張子は";"で区切ると複数登録できます。また、複数セット登録できます。
                return "画像ファイル(*.jpg,*.bmp,*.gif,*.png,*.tiff)|*.jpg;*.bmp;*.gif;*.png;*.tiff|JPEG(*.jpg)|*.jpg|ビットマップ(*.bmp)|*.bmp|GIF(*.gif)|*.gif|PNG(*.png)|*.png|TIFF(*.tiff)|*.tiff";
            }
        }

        /// <summary>
        /// ファイル名を取得します。
        /// </summary>
        public string FileName
        {
            get
            {
                return startFile;
            }
        }

        /// <summary>
        /// プラグインを初期化します。
        /// </summary>
        /// <param name="host">プロジェクトの情報。</param>
        public void Initialize(IHost host)
        {
            ho = host;
        }

        /// <summary>
        /// ファイルを読み込みます。
        /// </summary>
        /// <param name="file">読み込むファイル。</param>
        /// <returns>読み込み成功した場合はtrue、失敗した場合はfalseを返します。</returns>
        public bool LoadFile(string file)
        {
            string dir = Path.GetDirectoryName(file);
            string name = Path.GetFileNameWithoutExtension(file);
            Match match = Regex.Match(name, "[0-9]+", RegexOptions.None);
            if (match.Success)
            {
                int disit = match.Length;
                string baseName = name.Substring(0, name.Length - disit);
                int num;
                if (int.TryParse(name.Substring(match.Index, match.Length), out num))
                {
                    startFile = file;
                    string[] allFile = Directory.GetFiles(dir, "*" + Path.GetExtension(file));
                    List<string> temp = new List<string>();
                    for (int i = 0; i < allFile.Length; i++)
                    {
                        string s = Path.GetFileNameWithoutExtension(allFile[i]);
                        if (s.Contains(baseName))
                        {
                            match = Regex.Match(s, "[0-9]+", RegexOptions.None);
                            if (match.Success)
                            {
                                disit = match.Length;
                                int ind;
                                if (int.TryParse(s.Substring(match.Index, match.Length), out ind) && num <= ind)
                                {
                                    temp.Add(allFile[i]);
                                }
                            }
                        }
                    }
                    files = temp.ToArray();
                }
                else
                {
                    startFile = file;
                    files = new string[] { file };
                }
                try
                {
                    Bitmap image = new Bitmap(file);
                    baseSize = image.Size;
                    image.Dispose();
                    return true;
                }
                catch
                {
                    return false;
                }
            }
            else
            {
                try
                {
                    startFile = file;
                    files = new string[] { file };
                    Bitmap image = new Bitmap(file);
                    baseSize = image.Size;
                    image.Dispose();
                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 入力ファイルの情報を取得します。
        /// </summary>
        /// <returns>入力ファイルの情報。</returns>
        public MediaInfo GetMediaInfo()
        {
            if (files.Length > 1)
            {
                return new MediaInfo(files.Length, baseSize, MediaType.Video, 30);
            }
            else
            {
                return new MediaInfo(1, baseSize, MediaType.Image, 0);
            }
        }

        /// <summary>
        /// 入力ファイルのプロジェクト上での長さを取得します。
        /// </summary>
        /// <returns>ファイルの長さ。</returns>
        public int GetLength()
        {
            if (files.Length > 1)
            {
                return files.Length;
            }
            else
            {
                return (int)Math.Round(5 * ho.ProjectFrameRate, 0, MidpointRounding.AwayFromZero);
            }
        }

        /// <summary>
        /// 指定位置のフレームを取得します。
        /// </summary>
        /// <param name="index">フレームの位置。</param>
        /// <returns>指定位置のフレーム。</returns>
        public Bitmap GetFrame(int index)
        {
            //IMediaInput.GetMediaInfo()でMediaType.Audioを返している場合は使用されません。
            if (files.Length > 1)
            {
                if (index <= files.Length)
                {
                    int t = index;
                    while(t > -1 && !File.Exists(files[t]))
                    {
                        t--;
                    }
                    if (t < 0)
                    {
                        t = index;
                        while (t < files.Length && !File.Exists(files[t]))
                        {
                            t++;
                        }
                        if (t != files.Length)
                        {
                            using (Bitmap image = new Bitmap(files[t]))
                            {
                                if (baseSize.Equals(image.Size))
                                {
                                    if (image.PixelFormat != PixelFormat.Format32bppArgb)
                                    {
                                        return PluginUtils.ConvertBitmap(image);
                                    }
                                    else
                                    {
                                        return (Bitmap)image.Clone();
                                    }
                                }
                                else
                                {
                                    Bitmap canvas = new Bitmap(baseSize.Width, baseSize.Height, PixelFormat.Format32bppArgb);
                                    using (Graphics g = Graphics.FromImage(canvas))
                                    {
                                        g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                                        if (baseSize.Width >= baseSize.Height)
                                        {
                                            float h = (float)(baseSize.Width * image.Height / (double)image.Width);
                                            g.DrawImage(image, new RectangleF(new PointF(), new SizeF(baseSize.Width, h)));
                                        }
                                        else
                                        {
                                            float w = (float)(baseSize.Height * image.Width / (double)image.Height);
                                            g.DrawImage(image, new RectangleF(new PointF(), new SizeF(w, baseSize.Height)));
                                        }
                                    }
                                    return canvas;
                                }
                            }
                        }
                        else
                        {
                            return PluginUtils.NullImage(baseSize);
                        }
                    }
                    else
                    {
                        using (Bitmap image = new Bitmap(files[t]))
                        {
                            if (image.PixelFormat != PixelFormat.Format32bppArgb)
                            {
                                return PluginUtils.ConvertBitmap(image);
                            }
                            else
                            {
                                return (Bitmap)image.Clone();
                            }
                        }
                    }
                }
                else
                {
                    return PluginUtils.NullImage(baseSize);
                }
            }
            else if (File.Exists(startFile))
            {
                using (Bitmap image = new Bitmap(startFile))
                {
                    if (image.PixelFormat != PixelFormat.Format32bppArgb)
                    {
                        return PluginUtils.ConvertBitmap(image);
                    }
                    else
                    {
                        return (Bitmap)image.Clone();
                    }
                }
            }
            else
            {
                return PluginUtils.NullImage(baseSize);
            }
        }

        /// <summary>
        /// ビデオからオーディオを取得します。
        /// </summary>
        /// <returns>取得したオーディオ。</returns>
        public byte[] GetAudio()
        {
            //IMediaInput.GetMediaInfo()でMediaType.Videoを返した場合に使用されます。
            //オーディオのフォーマットはReferenceData.WaveFormatに変換して下さい。
            return null;
        }

        /// <summary>
        /// オーディオをフレーム単位で取得します。
        /// </summary>
        /// <param name="index">フレームの位置。</param>
        /// <returns>切り出したヘッダなしのデータ。</returns>
        public byte[] GetFrameAudio(int index)
        {
            //IMediaInput.GetMediaInfo()でMediaType.Audioを返した場合に使用されます。
            //面倒な場合はPluginUtils.TrimAudio()を使用して下さい。
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region IDisposable メンバ

        /// <summary>
        /// オブジェクトを破棄するときに呼ばれます。
        /// </summary>
        public void Dispose()
        {
            //Disposeすべきものを使用した場合、ここで必ずDisposeして下さい。
            //このメソッドはいつ呼ばれるかわからないので、想定外の場合でも例外をとばさず、正常に
            //破棄できるように設計して下さい。
        }

        #endregion
    }
}
