﻿using System;
using System.Collections.Generic;
using System.Text;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Utils;
using System.Drawing.Imaging;
using System.IO;
using System.Drawing;

namespace SamplePlugin
{
    public class SequentialImageOutput : IMediaOutput
    {
        string basePath = "";
        int digit = 0;
        int index;
        ImageFormat format = null;

        #region IMediaOutput メンバ

        /// <summary>
        /// プラグインの名前を取得します。
        /// </summary>
        public string PluginName
        {
            get
            {
                return "連番イメージ";
            }
        }

        /// <summary>
        /// 出力するファイルの名前を取得します。
        /// </summary>
        public string FileName
        {
            get
            {
                return basePath;
            }
        }

        /// <summary>
        /// 対応する拡張子を取得します。
        /// </summary>
        public string CorrespondingType
        {
            get
            {
                //書式は"表示名|拡張子"を一セットとし、拡張子は";"で区切ると複数登録できます。また、複数セット登録できます。
                return "ビットマップ(*.bmp)|*.bmp|JPEG(*.jpg)|*.jpg|GIF(*.gif)|*.gif|PNG(*.png)|*.png|TIFF(*.tiff)|*.tiff";
            }
        }

        /// <summary>
        /// 出力するファイルの形式を取得します。
        /// </summary>
        public MediaType InputType
        {
            get
            {
                //MediaType.Imageだとフレームのみ、MediaType.Audioだとオーディオのみ、MediaType.Videoだと両方が渡されます。
                //ただし、MediaType.Videoの場合、オーディオが無い場合はIMediaOutput.AddFrameAudio()は呼ばれません。
                return MediaType.Image;
            }
        }

        /// <summary>
        /// プラグインを初期化します。
        /// </summary>
        /// <param name="filename">出力するファイルの名前。</param>
        /// <param name="host">プロジェクトの情報。</param>
        public void Initialize(string filename, IHost host)
        {
            basePath = filename;
            digit = host.ProjectLength.ToString().Length;
            switch (Path.GetExtension(filename))
            {
                case ".bmp":
                    {
                        format = ImageFormat.Bmp;
                        break;
                    }
                case ".jpg":
                    {
                        format = ImageFormat.Jpeg;
                        break;
                    }
                case ".gif":
                    {
                        format = ImageFormat.Gif;
                        break;
                    }
                case ".tiff":
                    {
                        format = ImageFormat.Tiff;
                        break;
                    }
                default:
                    {
                        format = ImageFormat.Png;
                        break;
                    }
            }
        }

        /// <summary>
        /// データを保存する際のオプションを設定するウインドウを表示します。
        /// </summary>
        /// <param name="hwnd">メインウインドウのハンドル</param>
        /// <returns>設定が完了した場合はtrue、そうでない場合や、キャンセルした場合はfalseを返します。</returns>
        public bool ShowSaveOption(IntPtr hwnd)
        {
            //特に設定するものがない場合はそのままtrueを返してもかまいません。
            return true;
        }

        /// <summary>
        /// フレームを追加します。
        /// </summary>
        /// <param name="image">追加するフレーム。</param>
        public void AddFrameImage(Bitmap image)
        {
            //IMediaOutput.InputTypeでMediaType.Audioを返した場合使用されません。
            //保存するときは、このメソッドを用いて1枚ずつフレームが送られてきます。
            image.Save(Path.Combine(Path.GetDirectoryName(basePath), Path.GetFileNameWithoutExtension(basePath)) + index.ToString("d" + digit.ToString()) + Path.GetExtension(basePath), format);
            index++;
        }

        /// <summary>
        /// オーディオを追加します。
        /// </summary>
        /// <param name="audio">追加するヘッダ付きオーディオ。</param>
        public void AddFrameAudio(byte[] audio)
        {
            //IMediaOutput.InputTypeでMediaType.Imageを返した場合使用されません。
            //データは、一度に全て送られます。
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// 現在行われている書き込み処理を全て完了します。
        /// </summary>
        public void Close()
        {
            //このメソッドはファイル書き込みをキャンセルした場合にも呼ばれます。
            //ファイルストリームを開いている場合はここで閉じて下さい。
        }

        #endregion

        #region IDisposable メンバ

        /// <summary>
        /// オブジェクトを破棄するときに呼ばれます。
        /// </summary>
        public void Dispose()
        {
            //Disposeすべきものを使用した場合、ここで必ずDisposeして下さい。
            //このメソッドはいつ呼ばれるかわからないので、想定外の場合でも例外をとばさず、正常に
            //破棄できるように設計して下さい。
        }

        #endregion
    }
}
